import pathlib
import typing
import re

import lxml.etree as ET  # nosec
from defusedxml import lxml

from odd.typedefs import Element


def get_root(xml_filename: pathlib.Path):
    with xml_filename.open(mode="rb") as f:
        return lxml.parse(f).getroot()


def get_view_arch(record: Element) -> typing.Optional[Element]:
    for field in record.iterchildren(tag="field"):
        if field.get("name") == "arch":
            return field
    return None


def get_xpath_expr_target_element(
    xpath_expr: typing.Optional[str]
) -> typing.Optional[str]:
    if not xpath_expr:
        return None
    last_part = xpath_expr.split("/")[-1]
    m = re.search(r"(?P<modifier>[^:]+::)?(?<!@)(?P<nodename>[^\[]+)", last_part)
    nodename = None
    if m is not None:
        nodename = m.group("nodename")
    if nodename == ".":
        nodename = None
    return nodename


def drop_comments(iterable):
    for node in iterable:
        if node.tag is not ET.Comment:
            yield node
